'use strict';

const rp                  = require('request-promise');
const _                   = require('lodash');
const domain              = require('domain');
const AuthenticatedEntity = require('@codefresh-io/authenticated-entity').User;

function request(opt) {
    opt = opt || {};

    const serviceConfig   = opt.serviceConfig;
    if(!serviceConfig){
        throw new Error('Service config not provided');
    }
    const serviceBasePath = `${serviceConfig.protocol}://${serviceConfig.uri}:${serviceConfig.port}`;
    const path            = opt.path;
    const requestOptions  = _.get(opt, 'requestOptions', {});
    const headers = _.get(requestOptions, 'headers', {});

    const d                   = domain.active;
    const authenticatedEntity = _.chain(d).get('authenticatedEntity').value();
    if (authenticatedEntity) {
        const encodedEntity = AuthenticatedEntity.constructFromUserEntity(authenticatedEntity).encodeToBase64String();
        headers['X-Authenticated-Entity-Json'] = encodedEntity;
    }
    const requestId = _.chain(d).get('requestId').value();
    if(requestId){
        headers['X-Request-ID'] = requestId;
    }

    const options = {
        url: `${serviceBasePath}${path}`,
        resolveWithFullResponse: true,
        json: true,
        headers,
    };
    return rp(_.merge(options, requestOptions));
}

module.exports = request;
