'use strict';

/* eslint-disable no-unused-expressions */
/* eslint-disable no-undef */

const Promise    = require('bluebird'); // jshint ignore:line
const chai       = require('chai');
const _          = require('lodash');
const proxyquire = require('proxyquire');
const sinon      = require('sinon');
const sinonChai  = require('sinon-chai');
const request    = require('./../request');
const expect     = chai.expect;
chai.use(sinonChai);


describe('Http infra unit test', () => {

    function fakeAuthEntity() {
        return {
            _id: 'fake-id'
        }
    }

    function fakeAuthEntityCtor() {

        this.toJson = () => {
            return JSON.stringify(fakeAuthEntity());
        };

        this.encodeToBase64String = () => (new Buffer(this.toJson())).toString('base64');
        return this;
    }

    function fakeConstructFromUserEntity() {
        return new fakeAuthEntityCtor();
    }

    function fakeOptionsObject() {
        return {};
    }

    function fakeServiceConfig() {
        return {
            protocol: 'protocol',
            uri: 'uri',
            port: 'port'
        };
    }

    const expectedHeadersEntries = ['X-Authenticated-Entity-Json'];

    function prepareProxy(opt) {
        return proxyquire('./../request', _.merge(opt, {
            'domain': {
                active: {
                    authenticatedEntity: function () {return fakeAuthEntity();}
                }
            },
            '@codefresh-io/authenticated-entity': {
                User: {
                    constructFromUserEntity: fakeConstructFromUserEntity
                }
            }
        }));
    }

    it('Should prepare request', () => {
        function mockPromiseRequest(opt) {
            expect(opt).to.not.be.empty;
            expect(opt.resolveWithFullResponse).to.be.true;
            expect(opt.json).to.be.true;
            expect(opt.headers).to.have.keys(expectedHeadersEntries);
            expect(opt.headers['X-Authenticated-Entity-Json'])
                .to
                .be
                .deep
                .equal('eyJfaWQiOiJmYWtlLWlkIn0=');
            expect(JSON.parse(Buffer.from(opt.headers['X-Authenticated-Entity-Json'], 'base64')
                .toString())).to.be.deep.equal(fakeAuthEntity());

            return Promise.resolve({});
        }

        const requestProxy = prepareProxy({
            'request-promise': mockPromiseRequest
        });

        return requestProxy({
            requestOptions: fakeOptionsObject(),
            serviceConfig: fakeServiceConfig
        });
    });

    it('Should throw an error on failed request', () => {

        function mockPromiseRequest() {
            return Promise.reject({
                error: new Error('Native error should be wrapped'),
                message: 'Fake massage'
            });
        }

        const requestProxy = prepareProxy({
            'request-promise': mockPromiseRequest
        });

        return requestProxy({
            requestOptions: fakeOptionsObject(),
            serviceConfig: fakeServiceConfig
        })
            .catch((err) => {
                expect(err.error).to.be.an.instanceOf(Error);
            });

    });

    it('Should return the body of the response', () => {
        function mockPromiseRequest() {
            return Promise.resolve({
                body: {
                    key: 'val'
                }
            });
        }

        const requestProxy = prepareProxy({
            'request-promise': mockPromiseRequest
        });
        return requestProxy({
            requestOptions: fakeOptionsObject(),
            serviceConfig: fakeServiceConfig
        })
            .then((body) => {
                expect(body).to.not.be.empty;
            });
    });

});
